package com.vividsolutions.jump.workbench.ui.plugin.cormat;

/** 
 * This class represents a CorMat Correction Matrix data point.
 * Eash <code>CormatDataPoint</code> describes the planimetric correction (deltax, deltay)
 * to be applied to the location (easting, northing).
 */
public class CormatDataPoint {
	
	/**
	 *  The easting-coordinate.
	 */
	double easting;  // Easting-Coordinate
	
	/**
	 *  The northing-coordinate.
	 */
	double northing; // Northing-Coordinate
	
	/**
	 *  The planimetric easting correction.
	 */
    double deltax;	 // Delta X
    
	/**
	 *  The planimetric northing correction.
	 */
    double deltay;   // Delta Y 
    
    /**
     *  Constructs a default initialized <code>CormatDataPoint</code>.
     */    
    public CormatDataPoint(){
    	// Constructor.
        easting = (double)0.0;
        northing = (double)0.0;
        deltax = (double)0.0;
        deltay = (double)0.0;
    }

    /**
     *  Constructs a <code>CormatDataPoint</code> for location (e,n) with planimetric correction (dx,dy).
     *
     *@param  e  the easting-value
     *@param  n  the northing-value
     *@param  dx the planimetric easting correction
     *@param  dy the planimetric northing correection
     */	
    public CormatDataPoint(double e, double n, double dx, double dy){
    	// Constructor.
		this.easting = e;
		this.northing = n;
		this.deltax = dx;
		this.deltay = dy;
    }

    /**
     *  Returns the <code>CormatDataPoint</code> easting value
     *@return easting value
     */
	double getEasting(){
		return easting;
	}
	
    /**
     *  Returns the <code>CormatDataPoint</code> northing value
     *@return northing value
     */
	double getNorthing(){
		return northing;
	}
	
    /**
     *  Returns the <code>CormatDataPoint</code> easting planimetric correction value
     *@return easting planimetric correction alue
     */
	double getDeltaX(){
		return deltax;
	}
	
    /**
     *  Returns the <code>CormatDataPoint</code> northing planimetric correction value
     *@return northing planimetric correction alue
     */
	double getDeltaY()
	{
		return deltay;
	}
	
}
