package com.vividsolutions.jump.workbench.ui.plugin.cormat;

import java.io.*;

import javax.swing.*;

import java.awt.GridLayout;

import java.io.IOException;
import java.util.Iterator;

import com.vividsolutions.jts.geom.Coordinate;
import com.vividsolutions.jts.geom.Geometry;
import com.vividsolutions.jts.io.ParseException;
import com.vividsolutions.jump.feature.Feature;
import com.vividsolutions.jump.feature.FeatureCollection;
import com.vividsolutions.jump.io.DriverProperties;
import com.vividsolutions.jump.io.IllegalParametersException;
import com.vividsolutions.jump.workbench.model.Layer;
import com.vividsolutions.jump.workbench.model.StandardCategoryNames;
import com.vividsolutions.jump.workbench.plugin.PlugInContext;
import com.vividsolutions.jump.workbench.ui.GUIUtil;

import com.vividsolutions.jump.io.ShapefileReader;
import com.vividsolutions.jump.io.ShapefileWriter;

/** 
 * This class executes the CorMat Data Matrix corrections. 
 * 
 */
public class CormatExecuter{

	private FeatureCollection fc;  // Feature Collection
	
	private CormatData cd;		   // Cormat Data
	
    private CormatProgressDialog progressDlg;   // Progress Dialog

    private boolean wasCorrections = false;  // Was Corrections Flag
    
	public CormatExecuter() {
		// Empty 
	}
	
	public void initialize(PlugInContext context) throws Exception {
	}

	public boolean execute(PlugInContext context)
		throws ParseException, IOException {
		return execute(context, false, null, null, null);
	}


    /**
     *  To executee the CorMat data process on the cormatInput file with the
     *  cormatCormat file and writing the results to the cormatOutput file.
     *  If addOutAsALayerFlag is true, then create a new layer with the results  
     *
     *@param  addOutAsALayerFlag  add cormatOutput data as a layer flag  
     *@param  cormatInputFile  
     *@param  cormatCormatFile  
     *@param  cormatOutputFile  
     *@return <data>true</data> if OK; <data>false</data> otherwise
     *@throws ParseException, IOException
     */
	public boolean execute(PlugInContext context, boolean addOutAsALayerFlag, File cormatInputFile, File cormatCormatFile, File cormatOutputFile)
		throws ParseException, IOException {

		// Make sure inputFile specified
		if (cormatInputFile == null){
			// Display Error Message
			String title = "Cormat Error";
			String message = "No Input SHP file specified.";
			JOptionPane.showMessageDialog(null, message, title, JOptionPane.ERROR_MESSAGE);
			return false;
		}; /*if*/
		
		// Make sure cormatFile specified
		if (cormatCormatFile == null)
		{
			// Display Error Message
			String title = "Cormat Error";
			String message = "No Cormat Data file specified.";
			JOptionPane.showMessageDialog(null, message, title, JOptionPane.ERROR_MESSAGE);
			return false;
		}; /*if*/
		
		// Make sure outputFile specified
		if (cormatOutputFile == null)
		{
			// Display Error Message
			String title = "Cormat Error";
			String message = "No Output SHP file specified.";
			JOptionPane.showMessageDialog(null, message, title, JOptionPane.ERROR_MESSAGE);
			return false;
		}; /*if*/

	     progressDlg = new CormatProgressDialog(context);
	     
	     progressDlg.pack();
	     progressDlg.setResizable(false);
	     progressDlg.setBounds(0, 0, 300, 100);
         GUIUtil.centre(progressDlg, context.getWorkbenchFrame());
         progressDlg.updateProgressPanel(0);
         progressDlg.setVisible(true);
         progressDlg.paintComponents(progressDlg.getGraphics());
         
         // Load the In Shape File In
		 if (readShapeFileIn(context, cormatInputFile) == false)
		 {
     	    // Close the CORMAT Dialog Box
             progressDlg.dispose();
			// Return Not-OK 
		    return(false);	 
		 };

		 // Load the Correction Matrix data
         if (loadCorMatData(cormatCormatFile) == false)
       	 {
      	    // Close the CORMAT Dialog Box
             progressDlg.dispose();
 			// Return Not-OK 
 		    return(false);	 
       	 };		
		
         // Correct In File 
         if (cormatData() == false)
       	 {
      	    // Close the CORMAT Dialog Box
             progressDlg.dispose();
  			// Return Not-OK 
  		    return(false);	 
       	 };
         
         // Save the corrected data into the out Shape file 
         if (writeShapeFileOut(context, cormatOutputFile) == false)
         {
      	    // Close the CORMAT Dialog Box
             progressDlg.dispose();
   			// Return Not-OK 
   		    return(false);	 
         };

 	    // Close the Progress Dialog Box
         progressDlg.dispose();

         // Check if addOutAsALayer flag is true
         if (addOutAsALayerFlag == true)
         {
        	 Layer layer =
        		 context.addLayer(
        				 StandardCategoryNames.WORKING,
        				 cormatOutputFile.getName(), fc);
        	 
    	     layer.setDescription("[ CorMat'ed " + cormatInputFile.getName() + "]");
         }; /*if*/

         // Return OK
		 return(true);
		 
	}

	// Read the shape file in
	private boolean readShapeFileIn(PlugInContext context, File cormatInputFile){

		// Update Progress Dialog
		progressDlg.updateProgressLabel("Reading Shapefile ... ");
        progressDlg.paintComponents(progressDlg.getGraphics());

		DriverProperties ifp = new DriverProperties(); // In File Driver Properties
		ShapefileReader sfr = new ShapefileReader();
		
		ifp.setProperty("File", cormatInputFile.getPath());

		try {
				fc = sfr.read(ifp);
		}
		catch (IllegalParametersException e) {
			
			System.out.println("IllegalParametersExceptions: "+ e.getMessage());			
			
			// Display Error Message
			String title = "Cormat Error";
			String message = "Error reading SHP file.";
			JOptionPane.showMessageDialog(null, message, title, JOptionPane.ERROR_MESSAGE);
			
			// Return Not-OK
			return(false);
			
		}
		catch (Exception e){
			System.out.println("Exception: "+ e.getMessage());
			
			// Display Error Message
			String title = "Cormat Error";
			String message = "Error reading SHP file.";
			JOptionPane.showMessageDialog(null, message, title, JOptionPane.ERROR_MESSAGE);

			// Return Not-OK
			return(false);

		}
		
		// return OK
		return(true);
		
	}
	
	// Load the CorMat data
	private boolean loadCorMatData(File cormatInputFile){
		
		// Update Progress Dialog
		progressDlg.updateProgressLabel("Loading Correction Matrix Data file ... ");
        progressDlg.paintComponents(progressDlg.getGraphics());
		
		cd = new CormatData();		   
		try {
			if (cd.LoadCormatFile(cormatInputFile) != 0)
			{
				// Display Error Message
				String title = "Cormat Error";
				String message = "Invalid CorMat file.  Minimum Coordinate found inside file.";
				JOptionPane.showMessageDialog(null, message, title, JOptionPane.ERROR_MESSAGE);
			}; /*if*/
		}
		catch (IOException e)
		{
			System.out.println("Load Cormat Error: "+ e.getMessage());			
			
			// Display Error Message
			String title = "Cormat Error";
			String message = "Error reading CorMat file.";
			JOptionPane.showMessageDialog(null, message, title, JOptionPane.ERROR_MESSAGE);
			
			// Return Not-OK
			return(false);
			
		};
		
		// Return OK
		return(true);
	}
	
	// CorMat the data 
	private boolean cormatData(){

		int np; // Number of Points
		int cn; // Coordinate Number
		int fnum; // feature number
		
		Coordinate rc;

		// Update Progress Dialog
		progressDlg.updateProgressLabel("Applying CorMat Translation ... ");
		progressDlg.updateProgressSize(fc.size());
		progressDlg.setVisible(true);
        progressDlg.paintComponents(progressDlg.getGraphics());
		
		// Offsetting
		fnum = 0;
	    for (Iterator ia = fc.iterator(); ia.hasNext(); )
	    {
	    	
	    	// Update Progress Bar
	    	fnum = fnum + 1;
	    	progressDlg.updateProgressPanel(fnum);

	    	// Get next feature
	        Feature fa = (Feature) ia.next();
	        Geometry ga = fa.getGeometry();
	        np = ga.getNumPoints();
			Coordinate[] cl = new Coordinate[np];
			cl = ga.getCoordinates();
			for (cn = 0; cn < np; cn++)
			{

			   // CorMat the coordinate
			   rc = cd.cormatOffset(cl[cn]);
			   if ((cl[cn].x != rc.x) || (cl[cn].y != rc.y))
			   {
				   /* Check if wasCorrections is false */
				   if (wasCorrections == false)
				   {
					   // Set was Corrections flag to true
					   wasCorrections = true;
				   }; /*if*/
			   }; /*if*/
			   cl[cn].x = rc.x;
			   cl[cn].y = rc.y;
			   
			}; /*for*/
			
	    }; /*for*/

	    // Turn off progress bar
		progressDlg.setVisible(true);

	    // Check for Outliers 
	    if (cd.getWasOutlier() == true){
			// Display Warning Message
			String title = "Cormat Warning";
			String message = "Some data points were outside the CorMat matrix and were NOT shifted.";
			JOptionPane.showMessageDialog(null, message, title, JOptionPane.WARNING_MESSAGE);
	    }; /*if*/
	    	
	    // Return OK
	    return(true);
	    
	}

	// Write the resultant Shape file out
	private boolean writeShapeFileOut(PlugInContext context, File cormatOutputFile){
		
		// Update Progress Dialog
		progressDlg.updateProgressLabel("Writing Shapefile ... ");
        progressDlg.paintComponents(progressDlg.getGraphics());
		
		DriverProperties ofp = new DriverProperties(); // In File Driver Properties
		ShapefileWriter sfw = new ShapefileWriter();

		ofp.setProperty("File", cormatOutputFile.getPath());
	
		try {
			    sfw.write(fc, ofp);
		}
		catch (IllegalParametersException e) {
			System.out.println("IllegalParametersExceptions: "+ e.getMessage());			
			
			// Display Error Message
			String title = "Cormat Error";
			String message = "Error writing SHP file.";
			JOptionPane.showMessageDialog(null, message, title, JOptionPane.ERROR_MESSAGE);
			
			// Return Not-OK
			return(false);
			
		}
		catch (Exception e){
			System.out.println("Exception: "+ e.getMessage());
			
			// Display Error Message
			String title = "Cormat Error";
			String message = "Error writing SHP file.";
			JOptionPane.showMessageDialog(null, message, title, JOptionPane.ERROR_MESSAGE);
		
			// Return Not-OK
			return(false);
			
		}

		// Return OK
		return(true);
		
	}

	// Cormat Progress Dialog Box Class
    public static class CormatProgressDialog extends JDialog {
    	
    	/**
		 * 
		 */
		private static final long serialVersionUID = 704805971214141041L;
//        private PlugInContext context;    // Context
    	private int fcSize;			      // Feature Collection Size
        private JLabel progresslabel; 	  // Progress Label
		private JProgressBar progressbar; // Progress Bar
        
		// Cormat Progress Dialog 
        public CormatProgressDialog(PlugInContext context) {
          super(context.getWorkbenchFrame());
//            this.context = context;
            setTitle("CorMat Processing Information");

            initialize();
            
        }

        // Initalization 
        private void initialize() {
        	
            JPanel progressPanel = createProgressPanel();
            JPanel dialogPanel = new JPanel();

            dialogPanel.add(progressPanel);
            
            getContentPane().add(dialogPanel);
            
            
        }

        // Create the progress panel
        private JPanel createProgressPanel() {
        	
            JPanel progressPanel = new JPanel(new GridLayout(2,1));

            progresslabel = new JLabel("Progress Report", JLabel.CENTER);
            progressPanel.add(progresslabel);
            
        	int minimum = 0;    // Default Value
    		int maximum = 100;  // Default Value
    		progressbar = new JProgressBar(minimum, maximum);
    		progressbar.setStringPainted(true);
    		progressbar.setVisible(false);
            progressPanel.add(progressbar);
            
            return progressPanel;
            
        }
        
        // Update the progress panel
        public void updateProgressPanel(int p){
        	progressbar.setValue(p);
        	// Repaint only for a percentage change
        	if (p > 0)
        	{
        		double op = ((double)p - 1.0) / (double)fcSize * 100.0;
        		double np = (double)p / (double)fcSize * 100.0;
        		int oip = (int)op;
        		int nip = (int)np;
        		// See if old percentage not the same as new percentage
        		if (oip != nip)
        		{
        			// Update progressBar only if change in the progress bar
        			progressbar.paint(progressbar.getGraphics());
        		}; /*if*/
        	}; /*if*/
        }
        
        // Update the progress label
        public void updateProgressLabel(String s){
        	progresslabel.setText(s);
        }

        // Update the progres size
        public void updateProgressSize(int sz){
        	fcSize = sz;
        	progressbar.setMaximum(fcSize);
        }

    }	
}